﻿using System;
using System.Globalization;

namespace Урок_7._2
{
    /// <summary>
    /// Пример интерфейса. Все интерфейсы рекомендовано называть с буквы I.
    /// Правила присвоения имен такие же как и для классов.
    /// Область видимости аналогично классам.
    /// Содержать могут свойства, методы. НЕ могут содержать поля, конструкторы.
    /// </summary>
    interface ICandle
    {
        string Id { get; }
        double Open { get; }
        double Close { get; }
        double High { get; }
        double Low { get; }

        bool IsWhite();
        bool IsBlack();
        string ToString();
        string ToString(IFormatProvider format);
    }

    /// <summary>
    /// Интерфейс формирующий полную свечу. Просто дополняем уже существующий интерфейс
    /// </summary>
    interface IFullCandle : ICandle
    {
        double Volume { get; }
        DateTime OpenDate { get; }
        DateTime CloseDate { get; }
    }

    /// <summary>
    /// Интерфейс описывающий имя инструмента.
    /// </summary>
    interface ISecurityName
    {
        string SecurityName { get; }
    }

    /// <summary>
    /// Пример класса который наследует интерфейс. Наследование аналогично наследованию классов.
    /// Можно наследовать сразу несколько интерфейсов, но нельзя несколько классов сразу.
    /// </summary>
    class SampleFirst : ICandle
    {
        private double _open;

        public string Id { get; private set; }
        public double Open
        {
            get { return _open; }
        }
        public double Close { get; private set; }
        public double High { get; private set; }
        public double Low { get; private set; }
        public bool IsFirst
        {
            get { return true; }
        }


        public bool IsWhite()
        {
            return Close > Open;
        }

        public bool IsBlack()
        {
            return Open > Close;
        }

        public override string ToString()
        {
            return ToString(CultureInfo.CurrentCulture);
        }

        public string ToString(IFormatProvider format)
        {
            return string.Format(format, "Open: {0,5:0.##} | Close: {1,5:0.##} | High: {2,5:0.##} | Low: {3,5:0.##} | Id: {4}", Open, Close, High, Low, Id);
        }



        public SampleFirst(double open, double close, double high, double low, string id)
        {
            _open = open;
            Close = close;
            High = high;
            Low = low;
            Id = id;
        }
    }

    /// <summary>
    /// Второй класс который наследует полный интерфейс свечи.
    /// </summary>
    class SampleSecond : IFullCandle, ISecurityName
    {
        public string Id { get; private set; }
        public double Open { get; private set; }
        public double Close { get; private set; }
        public double High { get; private set; }
        public double Low { get; private set; }
        public string SecurityName { get; private set; }
        public bool IsSecond
        {
            get { return true; }
        }

        public double Volume { get; set; }
        public DateTime OpenDate { get; set; }
        public DateTime CloseDate { get; set; }


        public bool IsWhite()
        {
            return Close > Open;
        }

        public bool IsBlack()
        {
            return !IsWhite();
        }

        // Реализацию метода ToString() мы делать не будем. Она уже объявлена в классе object
        // поэтому в данном классе я могу ничего не создавать а пользоваться тем что уже есть
        // в родительском классе. Интерфейсу все равно где реализован метод, в дочернем или в 
        // родительском, главное чтобы реализация была.

        public string ToString(IFormatProvider format)
        {
            return string.Format(format, "Open: {0,5:0.##} | Close: {1,5:0.##} | High: {2,5:0.##} | Low: {3,5:0.##} | Id: {4}", Open, Close, High, Low, Id);
        }



        public SampleSecond(double open, double close, double high, double low, string id)
        {
            // В этом классе все свойства имеют set методы. И нет полей вообще.
            Open = open;
            Close = close;
            High = high;
            Low = low;

            Id = id;
        }
    }
}
